/* for gromacs 2.0 
 * extra routines for hole making using grasp surface 
 * or msms molecular surface (but keep "grasp" in routine names and stuff
 *  cos I can't be bothered to change them) 
 v3 introduce a command-line mdp file for hole params read with readhole */
/* (c) Graham Smith Sansom Group 1999, BMM Group ICRF 2001 / CR-UK 2002 */ 

/* 'global' variables to these grasp surf functions, invisible outside */ 

#include "hole.h"
#include "readinp.h"
#define LineLen 1000

  static rvec       *gsurfvert;
  static rvec       *gsurfnorm;
  static ivec       *gsurftri; 
  static int        *gsurfann=NULL;  
  static int        nv; 
  static int        ntri; 

/* end declaration of 'local global' variables */

void prv ( char *name, rvec r)
{
  fprintf(stderr," %s %f %f %f\n", name, r[XX], r[YY], r[ZZ]); 
}

void readmsms_ascii ( char *filenm, int natom) 
{
 /* new in v 2.0 */
 /* assumes filenm will be just the base part, not the .vert and .face extns*/ 
  FILE     *inpfl;
  char     line[STRLEN];
  char     complete_filenm[STRLEN]; 
  int      nred, i,j,idum,nsphere;
  real     rdum, rdensity, rprober; 

  /* allocate atom vertex neighbour list */
  printf("Reading ASCII msms surface\natomic surface: allocating list of nn vertex of each of %i atoms \n",natom);
  snew(gsurfann,natom);  

  /* get header info */
  strcpy(complete_filenm, filenm);
  strcat(complete_filenm, ".vert");
  inpfl=fopen(complete_filenm, "r");
  if (inpfl == NULL) {
    fatal_error(0,"failed to open file %s\n", complete_filenm); 
  }

  printf("atomic surface: reading data from file %s\n", complete_filenm); 
  for(i=0;i<2;i++) /* read empty lines at start */
  {
    fgets(line, STRLEN, inpfl);
  } 
  fgets(line, STRLEN, inpfl);
  nred = sscanf(line,"%i %i %f %f", &nv, &nsphere, &rdensity, &rprober); 
  /*  printf("%i %i %f %f\n", nv, nsphere, rdensity, rprober); 
      printf("read %i data\n", nred); */ 

  printf("atomic surface: reading %i vertices and normals...",nv); 
  fflush(stdout);
 
  /* allocate arrays for holding surface information */
  snew(gsurfvert,nv); 
  snew(gsurfnorm,nv); 

  /* start vertices from 0 even though from 1 in file */
  /* address element as, eg, gsurfvert[i][XX] */

  /* read vertices and normals */
  for(i=0;i<nv;i++)
  {
    fgets(line, STRLEN, inpfl);
    nred = sscanf(line,"%f %f %f %f %f %f", 
	   &gsurfvert[i][XX], &gsurfvert[i][YY], &gsurfvert[i][ZZ], 
	   &gsurfnorm[i][XX], &gsurfnorm[i][YY], &gsurfnorm[i][ZZ]);
    if(nred != 6) {
      fatal_error(0,"atomic surface: read error for vertices for #%i\n",i+1);
    }
    /* convert to nm */
    for(j=0;j<3;j++)  gsurfvert[i][j] /= 10;

#ifdef DEBUG
    printf(" %f %f %f  %f %f %f\n", 
	   gsurfvert[i][XX], gsurfvert[i][YY], gsurfvert[i][ZZ],
	   gsurfnorm[i][XX], gsurfnorm[i][YY], gsurfnorm[i][ZZ]);
#endif 
  }
  fclose(inpfl); 
  printf("done\n");fflush(stdout);

  /* finally the triangles (even though I don't use them) */

  strcpy(complete_filenm, filenm);
  strcat(complete_filenm, ".face");
  inpfl=fopen(complete_filenm, "r");
  if (inpfl == NULL) {
    fatal_error(0,"failed to open file %s\n", complete_filenm); 
  }
  printf("atomic surface: reading data from file %s\n", complete_filenm); 
  for(i=0;i<2;i++) /* read empty lines at start */
  {
    fgets(line, STRLEN, inpfl);
  } 
  fgets(line, STRLEN, inpfl); 
  nred = sscanf(line,"%i %i %f %f", &ntri, &nsphere, &rdensity, &rprober); 
  /* printf("%i %i %f %f\n", ntri, nsphere, &rdensity, &rprober); */
  printf("atomic surface: reading %i trangles...",ntri); 
  fflush(stdout);

  /* allocate arrays */ 
  snew(gsurftri,ntri); 

  for(i=0;i<ntri;i++)
  {
    fgets(line, STRLEN, inpfl);
    nred = sscanf(line,"%i %i %i", 
	   &gsurftri[i][XX], &gsurftri[i][YY], &gsurftri[i][ZZ]);
    if(nred != 3) 
      printf("atomic surface: read error for triangles for #%i\n",i+1);
    for(j=0;j<3;j++)  gsurftri[i][j]--;
  }

  fclose(inpfl); 
  printf("done\n");fflush(stdout);

#ifdef DEBUG
  for (i=0;i<100;i++)
     printf("%f %f %f \n%f %f %f \n%i %i %i \n", 
         gsurfvert[i][XX], gsurfvert[i][YY], gsurfvert[i][ZZ], 
         gsurfnorm[i][XX], gsurfnorm[i][YY], gsurfnorm[i][ZZ], 
         gsurftri[i][XX], gsurftri[i][YY], gsurftri[i][ZZ]);
#endif
}


/* from readgrasp.c */

void readgrasp_ascii ( char *filenm, int natom) 
{
  FILE     *inpfl;
  char     line[STRLEN];
  int      nred, i,j,idum;
  real     rdum; 

  /* allocate atom vertex neighbour list */
  printf("Reading ASCII grasp surface\natomic surface: allocating list of nn vertex of each of %i atoms \n",natom);
  snew(gsurfann,natom);  

  /* get header info */
  inpfl=fopen(filenm, "r");
  if (inpfl == NULL) {
    fatal_error(0,"failed to open atomic surface file %s\n", filenm); 
  }

  printf("atomic surface: reading surface data from file %s\n", filenm); 
  nred = 0 ;
  while (nred != 4) /* read header info  */
  {
    fgets(line, STRLEN, inpfl);
    nred=sscanf(line,"%i %i %i %f", &nv, &ntri, &idum, &rdum); 
  } 

  printf("atomic surface: reading %i vertices and normals and %i triangles...",
      nv, ntri); fflush(stdout);
  /* skip next line or causes confusion...*/ 
  fgets(line, STRLEN, inpfl);
 
  /* allocate arrays for holding surface information */
  snew(gsurfvert,nv); 
  snew(gsurfnorm,nv); 
  snew(gsurftri,ntri); 
  /* start vertices from 0 even though from 1 in file */
  /* address element as, eg, gsurfvert[i][XX] */

  /* read vertices */
  for(i=0;i<nv;i++)
  {
    nred=0 ; 
    while(nred != 4) {
      fgets(line, STRLEN, inpfl);
      nred=sscanf(line,"%i %f %f %f", &idum, &gsurfvert[i][XX], 
		  &gsurfvert[i][YY], &gsurfvert[i][ZZ]);
    }
    /* convert to nm */
    gsurfvert[i][XX] /= 10;
    gsurfvert[i][YY] /= 10;
    gsurfvert[i][ZZ] /= 10;
    if(idum != i+1) {
      fatal_error(0,"atomic surface: read error for vertices for i %i line %i\n",i,idum);
    }
#ifdef DEBUG
    printf("%s\n", line); 
    printf("%i %i - %i %f %f %f\n", i, nred, idum, gsurfvert[i][XX], 
       gsurfvert[i][YY], gsurfvert[i][ZZ]);
#endif
  }

  /* read over accessible surface points - should be same as vertices */
  for(i=0;i<nv;i++)
    fgets(line, STRLEN, inpfl);
    sscanf(line,"%i %f %f %f", &idum, &rdum, &rdum, &rdum ); 

  /* read normals  */
  for(i=0;i<nv;i++)
  {
    fgets(line, STRLEN, inpfl);
    sscanf(line,"%i %f %f %f", &idum, &gsurfnorm[i][XX], &gsurfnorm[i][YY], 
       &gsurfnorm[i][ZZ]);
    if(idum != i+1) {
      fatal_error(0,"atomic surface: read error for normals for #%i\n",idum);
    }
#ifdef DEBUG
    printf("%i %f %f %f\n", idum, gsurfnorm[i][XX], gsurfnorm[i][YY], 
       gsurfnorm[i][ZZ]);
#endif
  }

  /* finally the triangles */
  for(i=0;i<ntri;i++)
  {
    fgets(line, STRLEN, inpfl);
    sscanf(line,"%i %i %i %i", &idum, &gsurftri[i][XX], &gsurftri[i][YY], 
       &gsurftri[i][ZZ]);
    if(idum != i+1) printf("atomic surface: read error for triangles for #%i\n",idum);
    for(j=0;j<3;j++)  gsurftri[i][j]--;
  }

  fclose(inpfl); 
  printf("done\n");fflush(stdout);

#ifdef DEBUG
  for (i=0;i<100;i++)
     printf("%f %f %f \n%f %f %f \n%i %i %i \n", 
         gsurfvert[i][XX], gsurfvert[i][YY], gsurfvert[i][ZZ], 
         gsurfnorm[i][XX], gsurfnorm[i][YY], gsurfnorm[i][ZZ], 
         gsurftri[i][XX], gsurftri[i][YY], gsurftri[i][ZZ]);
#endif
}

/* from read_bin_grasp_surf.c */

void readgrasp_bin ( char *filenm, int natom) 
/* uses calls to fread that I got out of ReadGrasp.c by Martin Noble;
his files are in gromacs/grasp_surf/martin/ */
{
  FILE     *inpfl;
  int      i;
  real     cmx,cmy,cmz;

  /* mno */
  char buffer[360];
  int j;
  int hasVertices, hasAccessibles, hasNormals, hasPotential; 
  int hasTriangles, hasProperty; 
  char *cPntr;
  size_t bytes_read;
  /* end mno */

  /* allocate atom vertex neighbour list */
  printf("Reading binary grasp surface\natomic surface: allocating list of nn vertex of each of %i atoms \n",natom);
  snew(gsurfann,natom);  

  /* get header info */ 
  inpfl=fopen(filenm, "r");
  if (inpfl == NULL) {
    fatal_error(0,"failed to open atomic surface file %s\n", filenm); 
  }
  printf("atomic surface: reading surface data from file %s\n", filenm); 

  /* mno */

  fseek (inpfl, 4, SEEK_SET);
  fread (buffer, 1, 64, inpfl); buffer[63] = '\0';
  if (strncmp(buffer,"format=2",8)){
    fatal_error(0,"Confused by this format I fear \n");
  }
  fseek (inpfl,  92, SEEK_SET);
  
  fread (buffer, 1, 64, inpfl); buffer[63] = '\0';
  hasVertices = (strstr(buffer,"vertices")!=NULL);
  hasAccessibles = (strstr(buffer,"accessibles")!=NULL);
  hasNormals = (strstr(buffer,"normals")!=NULL);
  hasTriangles = (strstr(buffer,"triangles")!=NULL);
  
  if(!hasVertices){
    fatal_error(0,"No vertices found in file \n");
  }
  if(!hasAccessibles){
    fatal_error(0,"No Accessibles found in file \n");
  }
  if(!hasNormals){
    fatal_error(0,"No Normals found in file \n");
  }
  if(!hasTriangles){
    fatal_error(0,"No Triangles found in file \n");
  }

  fseek (inpfl, 180, SEEK_SET);
  fread (buffer, 1, 64, inpfl); buffer[63] = '\0';
  hasPotential = (strstr(buffer,"potential")!=NULL);
  hasProperty = (strstr(buffer,"gproperty1")!=NULL);
  
  fseek (inpfl, 268, SEEK_SET);
  fread (buffer, 1, 64, inpfl); buffer[63] = '\0';
  cPntr = strtok(buffer," "); nv = atoi (cPntr);
  cPntr = strtok(NULL," "); ntri = atoi (cPntr);

  fseek (inpfl, 356, SEEK_SET);
  fread (buffer, 1, 64, inpfl); buffer[63] = '\0';
  cPntr = strtok(buffer," "); cmx = (atof (cPntr))/10;
  cPntr = strtok(NULL," "); cmy = (atof (cPntr))/10;
  cPntr = strtok(NULL," "); cmz = (atof (cPntr))/10;
  printf ("atomic surface: CM of surface is at %f %f %f\n", cmx, cmy, cmz);  
  fseek (inpfl, 444, SEEK_SET);

  printf("atomic surface: reading %i vertices and normals and %i triangles\n", nv, ntri); 
  

  /* end mno */ 


  /* allocate arrays for holding surface information */
  snew(gsurfvert,nv); 
  snew(gsurfnorm,nv); 
  snew(gsurftri,ntri); 
  /* start vertices from 0 even though from 1 in file */
  /* address element as, eg, gsurfvert[i][XX] */

  /* read vertices */
  if (hasVertices){
    bytes_read = fread (gsurfvert, 1, nv*(sizeof(rvec)), inpfl);
    if(bytes_read != nv*(sizeof(rvec))){
      fatal_error(0,"atomic surface: read only %i bytes for vertices\n",
		  bytes_read);
    }else{
      printf("    read %i vertices successfully\n", nv);  
    } 
    fseek (inpfl, 8, SEEK_CUR);
    /* MN needed to reform_sgi_float here but I don't seem to */ 
  }

  for(i=0;i<nv;i++)
  {
    /* convert to nm */
    gsurfvert[i][XX] /= 10;
    gsurfvert[i][YY] /= 10;
    gsurfvert[i][ZZ] /= 10;
#ifdef DEBUG
    printf("%i %f %f %f\n", i, gsurfvert[i][XX], gsurfvert[i][YY], 
       gsurfvert[i][ZZ]);
#endif
  }

  /* read over accessible surface points - should be same as vertices */
  if (hasAccessibles){
    bytes_read = fread (gsurfnorm, 1, nv*(sizeof(rvec)), inpfl);
    if(bytes_read != nv*(sizeof(rvec))){
      fatal_error(0,"atomic surface: read only %i bytes for accessibles\n",
		  bytes_read);
    }else{
      printf("    skipped %i accessibles successfully\n", nv);  
    }     
    fseek (inpfl, 8, SEEK_CUR);
  }

  /* read normals  */
  if (hasNormals){
    bytes_read = fread (gsurfnorm, 1, nv*(sizeof(rvec)), inpfl);
    if(bytes_read != nv*(sizeof(rvec))){
      fatal_error(0,"atomic surface: read only %i bytes for normals\n",
		  bytes_read);
    }else{
      printf("    read %i normals successfully\n", nv);  
    }     
    fseek (inpfl, 8, SEEK_CUR);
  }

#ifdef DEBUG
  for(i=0;i<nv;i++)
    printf("%i %f %f %f\n", i, gsurfnorm[i][XX], gsurfnorm[i][YY], 
       gsurfnorm[i][ZZ]);
#endif

  /* and finally triangles */ 
  if (hasTriangles){
    bytes_read = fread (gsurftri, 1, ntri*(sizeof(ivec)), inpfl);
    if(bytes_read != ntri*(sizeof(ivec))){
      fatal_error(0,"atomic surface: read only %i bytes for triangles\n",
		  bytes_read);
    }else{
      printf("    read %i triangles successfully\n", ntri);  
    }     
    fseek (inpfl, 8, SEEK_CUR);
  }

  for(i=0;i<ntri;i++){
    for(j=0;j<3;j++)  gsurftri[i][j]--;
#ifdef DEBUG
    printf("%i %i %i %i\n", i, gsurftri[i][XX], gsurftri[i][YY], 
       gsurftri[i][ZZ]);
#endif 
  }

  fclose(inpfl); 
  printf("done\n");fflush(stdout);

#ifdef DEBUG
  for (i=0;i<100;i++)
     printf("%f %f %f \n%f %f %f \n%i %i %i \n", 
         gsurfvert[i][XX], gsurfvert[i][YY], gsurfvert[i][ZZ], 
         gsurfnorm[i][XX], gsurfnorm[i][YY], gsurfnorm[i][ZZ], 
         gsurftri[i][XX], gsurftri[i][YY], gsurftri[i][ZZ]);
#endif

}




/* from atomnnlist.c */

void build_grasp_nn_list(int firstatom, int lastatom, rvec coord[],int toupd)
{
/* if have done toupd calls since last update, regenerates the list of
which gsurf vertex is closest neighbour, for each atom. */ 

/* just finds a single nn vertex for each atom. ~/c/graspsurf/longnlists 
has routines for building a proper vertex neighbour list for each atom, but
it wasn't worth the bother. Good practice at qsort though. */


  int ia, iv, minind=0; 
  /* int j, jmax, it; */ 
  real minr2,r2;
  rvec deltar;

  /* count-calls stuff */
  static int ictcall=0;
  ictcall ++;
  if (ictcall > toupd) ictcall=1;
  if (ictcall == 1)
  {
    /* now (re)build neighb list */
    fprintf(stdlog, "atomic surface: building atom-vertex nn list...\n");
    for(ia=firstatom; ia<lastatom;ia++)
    {
      minr2=10000000.0;
      for(iv=0; iv<nv; iv++) /* find all r(ia-iv) for this ia */
      {
	rvec_sub(coord[ia],gsurfvert[iv],deltar); 
	r2 = norm2(deltar);
        if (r2 < minr2) 
        {
          minr2 = r2; 
          minind= iv;
        }
      }
      gsurfann[ia]=minind; 
#ifdef DEBUG
     printf("%6i %5.2f %5.2f %5.2f %6i %5.2f %5.2f %5.2f   %5.2f\n",
           ia, coord[ia][XX], coord[ia][YY], coord[ia][ZZ],  
           minind, gsurfvert[minind][XX], gsurfvert[minind][YY], 
           gsurfvert[minind][ZZ],sqrt(minr2));
#endif 
    }
/*    printf("done\n");fflush(stdout); */
  } /* end if...need to update */
}



/* from surfforce.c */

real  get_grasp_surf_force(int ia, rvec coor, real offset, rvec force)
{
/* finds force on atom ia ; also returns distance  from surface if inside it*/

  int ix;
  real a,modd;
  rvec deltar, sv, nhat, offvec;

#ifdef DEBUG
  prv("coor", coor);
#endif
  ix = gsurfann[ia]; 
  copy_rvec(gsurfvert[ix],sv); /* put the nn surf vertex in sv */
  copy_rvec(gsurfnorm[ix],nhat); /* and get the normal */
#ifdef DEBUG
  prv("norm", nhat);
#endif
  svmul(offset,nhat,offvec);
  rvec_inc(sv,offvec);  /* s -> s + offset*n  ; this is nn surf vertex + offset in dirn of normal*/
  rvec_sub(sv,coor,deltar);  /* d = s - r(ia) ; vector between atom and nn surf vertex */
  a=iprod(deltar,nhat);  /* a = d.n */
  if (a <= 0.0) /* atom is outside */
  {
    clear_rvec(force); 
    return 0.0;
  }
  else
  {
    modd=norm(deltar) + 0.00001; /* bit on end just stops divide by zero */
    copy_rvec(nhat, deltar); /* force in dirn of norm not vertex-atom vec */
    svmul((1.0/norm(deltar)),deltar,force); 
    /* force of modulus 1; multiply by extra force const once returned to main prog */
    return modd; 
  }     
}  

void average_surface_forces_over_residues
(t_atoms *atoms, rvec cfarr[], int start, int homenr)
{
  int n, i, ires, lastires, nat, firstat; 
  real weeny=0.0001; 
  rvec sumcf; 

  ires = lastires = atoms->atom[start].resnr ; 
  firstat = start ; 
  nat = 0 ; 
  clear_rvec(sumcf) ; 
  for (n=start; n<start+homenr; n++) {  
    ires = atoms->atom[n].resnr ; 
    if ( ires != lastires ) {
      /* new residue */ 
      project_last_residues_forces(nat,sumcf,cfarr,firstat,n); 
      /* housekeeping */
      nat = 0; 
      clear_rvec(sumcf); 
      lastires = ires ; 
      firstat = n ; 
    } else {   
      if ( norm (cfarr[n]) > weeny) {
	rvec_inc(sumcf,cfarr[n]); 
	nat ++ ; 
      }
    }
  }
  project_last_residues_forces(nat,sumcf,cfarr,firstat,start+homenr); 
}

void project_last_residues_forces(int nat, rvec sumcf, rvec cfarr[], int firstat, int n)
{
  int i;
  rvec avcf ; 
  real weeny=0.0001; 

  if ( nat > 0 ) { 
    svmul((1.0/nat), sumcf, avcf) ;
    /* set force to average force within residue over those atoms inside */ 
    for (i = firstat; i<n ; i++) {
      if ( norm (cfarr[i]) > weeny) {
	copy_rvec(avcf, cfarr[i]) ; 
      }
    }
  }
}

void free_grasp_surf_arrays(void)
{
  /* tidy up */
  sfree(gsurfvert); sfree(gsurfnorm);  sfree(gsurfann); sfree(gsurftri); 

}

void output_surf_as_pdb( void )
{
  /* output molec surface as a pdb (molsurfpdb.pdb). All as dummy atoms, 
     residue type MSF. CMS are the positions of the vertices in the
     molec surface, and the NMS are dummy atoms displaced from
     each vertex along the normal vector at that vertex. */ 

  static char *pdbformat="%-6s%5i  %-4s%-4s%5i    %8.3f%8.3f%8.3f%6.2f%6.2f      %4s\n";
  FILE *outsurf ; 
  int i, iat; 
  rvec vert_ang, vert_p_norm; 

  outsurf=fopen("molsurfpdb.pdb","w");
  if (outsurf == NULL )fatal_error(0,"Couldn't open molsurfpdb.pdb"); 

  iat = 1 ; 
  for(i=0;i<nv;i++)
  {
    /* vert_ang = 10 * gsurfvert */ 
    svmul(10.0, gsurfvert[i], vert_ang);  
    /* vert_p_normcc = vert + norm */
    rvec_add(vert_ang, gsurfnorm[i], vert_p_norm);  
    fprintf(outsurf, pdbformat, "ATOM", iat, "CMS", "MSF", i+1, 
	    vert_ang[XX], vert_ang[YY], vert_ang[ZZ], 
	    1.0, 1.0, "TEST" );
    fprintf(outsurf, pdbformat, "ATOM", iat, "NMS", "MSF", i+1, 
	    vert_p_norm[XX], vert_p_norm[YY], vert_p_norm[ZZ], 
	    1.0, 1.0, "TEST" );
    iat ++ ; 
  }
  fclose(outsurf); 
 
}

/* added for v3 - based on get_ir() in readir.c */ 
void readhole(t_hole *hole, char *hole_param_file)
{
  int       i,m,ninp,ecm_mode;
  t_inpfile *inp;
  char      *tmp;
  char      dummy[STRLEN];
  int       *nerror ; 

  snew(nerror,1); 
  *nerror=0; 

  inp=read_inpfile(hole_param_file,&ninp);

  CCTYPE ("HOLE PARAMETERS");
  
  CTYPE ("type of hole - Cyl, Grasp_bin, Grasp_ascii or MSMS"); 
  EETYPE("holetype",    hole->type,     ehole_names, nerror, TRUE);
  CTYPE ("radius of hole (cyl mode only)") ; 
  RTYPE ("hr",		hole->r,	0); 
  CTYPE ("x crd of centre of hole (cyl mode only)") ; 
  RTYPE ("hx",		hole->c[0],	0);
  CTYPE ("y crd of centre of hole (cyl mode only)") ; 
  RTYPE ("hy",		hole->c[1],	0); 
  CTYPE ("approximate z crd of centre of bilayer (needed for additional perpendicular repulsive force)") ; 
  RTYPE ("hz",		hole->c[2],	0); 
  CTYPE ("force constant for outward (hole making) force") ; 
  RTYPE ("hfm",		hole->lf,	50); 
  CTYPE ("In cyl mode, only atoms between hp1 and hp2 feel the hole-making repulsion. In mol surface modes, atoms between hp1 and hp2 have their outward force projected into the (x,y)-plane. Atom numbers start from 1"); 
  ITYPE ("hp1",         hole->lip1,     0);  
  ITYPE ("hp2",         hole->lip2,     0); 
  CTYPE ("Atoms between s1 and s2 feel an additional perpendicular repulsion"); 
  ITYPE ("s1",          hole->sol1,     0);  
  ITYPE ("s2",          hole->sol2,     0);  
  CTYPE ("z crds of lower (hz1) and upper (hz2) boundaries for perpendicular repulsion") ; 
  ITYPE ("hz1",         hole->z1,       0);  
  ITYPE ("hz2",         hole->z2,       0);  
  CTYPE ("force constant for perpendicular repulsion") ; 
  RTYPE ("sfm",		hole->sf,	50); 
  
  CTYPE ("frequency of updating atom-molecular surface neighbour list (not used in cyl mode)"); 
  ITYPE ("supf",        hole->supf,     10); 
  CTYPE ("offset (nm) of molecular surface used relative to one read, -ve = inward (not used in cyl mode)") ; 
  RTYPE ("sofs",	hole->sofs,	0.0); 
  snew(hole->outfnm,STRLEN); 
  CTYPE ("Log file for data about atoms within molecular surface"); 
  STYPE ("molsurf_log",	hole->outfnm,	"grasp.log");
  snew(hole->surffnm,STRLEN); 
  CTYPE ("File name of msms molecular surface file (in MSMS mode, just the base part, i.e. WITHOUT .vert or .face extns)"); 
  STYPE ("molsurf_file",hole->surffnm,	"NULL");
  CTYPE ("Debug surface forces by printing out insidegr.pdb showing forces on atoms, and molsurfpdb.pdb showing the surface. Stops at step 1"); 
  EETYPE("debugsurf",   hole->debug_surf,  yesno_names, nerror, TRUE);
  CTYPE ("Average hole-making forces over all atoms in a residue, rather than having a separate force on each atom"); 
  EETYPE("resforces",   hole->resforces,   yesno_names, nerror, TRUE);

  /* change atom numbers from 1-based to 0-based */ 
  /* defaults become -1 => skip the sections */ 
  hole->lip1 -- ; 
  hole->lip2 -- ; 
  hole->sol1 -- ; 
  hole->sol2 -- ; 

  /* rather ugly copying of information here */ 
  (hole->type == eholeCYL )    ? (hole->cyl = TRUE) : (hole->cyl = FALSE) ; 
  (hole->type == eholeBGRASP ) ? (hole->grasp_bin = TRUE) : (hole->grasp_bin = FALSE) ; 
  (hole->type == eholeAGRASP ) ? (hole->grasp_ascii = TRUE) : (hole->grasp_ascii = FALSE) ; 
  (hole->type == eholeMSMS )   ? (hole->msms_ascii = TRUE) : (hole->msms_ascii = FALSE) ; 

  /* print out the mdp again to processsed_(mdp) */ 

  sprintf(dummy, "processed_%s", hole_param_file) ; 
  write_inpfile(dummy,ninp,inp);
  for (i=0; (i<ninp); i++) {
    sfree(inp[i].name);
    sfree(inp[i].value);
  }
  sfree(inp);
}

void inithole(t_hole *hole, int natoms) 
{
  /* get surface */
  if(hole->msms_ascii)
    readmsms_ascii(hole->surffnm,natoms);
  if(hole->grasp_ascii)
    readgrasp_ascii(hole->surffnm,natoms);
  if(hole->grasp_bin)
    readgrasp_bin(hole->surffnm,natoms);
  /* open log */ 
  hole->outfp=ffopen(hole->outfnm, "w");
  if (hole->outfp == NULL) {
    fatal_error(0,"failed to open hole log file file %s\n", hole->outfnm); 
  }
  fprintf(hole->outfp, "# Time : atoms w/ non-zero hole-making force : depth inside of deepest\n");

}

